@@ -1,9 +1,6 @@
 // Copyright 2018-2021 the Deno authors. All rights reserved. MIT license.
 
-import {
-  assert,
-  assertEquals,
-} from "../../../test_util/std/testing/asserts.ts";
+import { assert } from "../../../test_util/std/testing/asserts.ts";
 import * as colors from "../../../test_util/std/fmt/colors.ts";
 export { colors };
 import { resolve } from "../../../test_util/std/path/mod.ts";
@@ -42,7 +39,7 @@ interface UnitTestOptions {
   permissions?: UnitTestPermissions;
 }
 
-type TestFunction = () => void | Promise<void>;
+type TestFunction = (tester: Deno.TestContext) => void | Promise<void>;
 
 export function unitTest(fn: TestFunction): void;
 export function unitTest(options: UnitTestOptions, fn: TestFunction): void;
