@@ -14,7 +14,10 @@ use crate::fs_util::{collect_files, is_supported_ext};
 use crate::tools::fmt::run_parallelized;
 use crate::{colors, file_watcher};
 use deno_ast::MediaType;
-use deno_core::error::{anyhow, generic_error, AnyError, JsStackFrame};
+use deno_core::anyhow::anyhow;
+use deno_core::error::generic_error;
+use deno_core::error::AnyError;
+use deno_core::error::JsStackFrame;
 use deno_core::serde_json;
 use deno_lint::diagnostic::LintDiagnostic;
 use deno_lint::linter::Linter;
